package sun.security.krb5.internal;

import sun.misc.HexDumpEncoder;
import sun.security.krb5.internal.util.KerberosString;
import java.util.Vector;
import sun.security.util.DerInputStream;
import sun.security.util.DerOutputStream;
import java.io.IOException;
import sun.security.krb5.Asn1Exception;
import sun.security.util.DerValue;

public class PAData
{
    private int pADataType;
    private byte[] pADataValue;
    private static final byte TAG_PATYPE = 1;
    private static final byte TAG_PAVALUE = 2;
    
    private PAData() {
        this.pADataValue = null;
    }
    
    public PAData(int paDataType, byte[] array) {
        this.pADataValue = null;
        this.pADataType = paDataType;
        if (array != null) {
            this.pADataValue = (byte[])array.clone();
        }
    }
    
    public Object clone() {
        PAData paData = new PAData();
        paData.pADataType = this.pADataType;
        if (this.pADataValue != null) {
            paData.pADataValue = new byte[this.pADataValue.length];
            System.arraycopy(this.pADataValue, 0, paData.pADataValue, 0, this.pADataValue.length);
            return paData;
        }
        return paData;
    }
    
    public PAData(DerValue derValue) throws Asn1Exception, IOException {
        this.pADataValue = null;
        if (derValue.getTag() != 48) {
            throw new Asn1Exception(906);
        }
        DerValue derValue2 = derValue.getData().getDerValue();
        if ((derValue2.getTag() & 0x1F) == 0x1) {
            this.pADataType = derValue2.getData().getBigInteger().intValue();
            DerValue derValue3 = derValue.getData().getDerValue();
            if ((derValue3.getTag() & 0x1F) == 0x2) {
                this.pADataValue = derValue3.getData().getOctetString();
            }
            if (derValue.getData().available() > 0) {
                throw new Asn1Exception(906);
            }
            return;
        }
        else {
            throw new Asn1Exception(906);
        }
    }
    
    public byte[] asn1Encode() throws Asn1Exception, IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        DerOutputStream derOutputStream2 = new DerOutputStream();
        derOutputStream2.putInteger(this.pADataType);
        derOutputStream.write(DerValue.createTag((byte)(-128), true, (byte)1), derOutputStream2);
        DerOutputStream derOutputStream3 = new DerOutputStream();
        derOutputStream3.putOctetString(this.pADataValue);
        derOutputStream.write(DerValue.createTag((byte)(-128), true, (byte)2), derOutputStream3);
        DerOutputStream derOutputStream4 = new DerOutputStream();
        derOutputStream4.write((byte)48, derOutputStream);
        return derOutputStream4.toByteArray();
    }
    
    public int getType() {
        return this.pADataType;
    }
    
    public byte[] getValue() {
        return (this.pADataValue == null) ? null : ((byte[])this.pADataValue.clone());
    }
    
    public static PAData[] parseSequence(DerInputStream derInputStream, byte b, boolean b2) throws Asn1Exception, IOException {
        if (b2 && ((byte)derInputStream.peekByte() & 0x1F) != b) {
            return null;
        }
        DerValue derValue = derInputStream.getDerValue().getData().getDerValue();
        if (derValue.getTag() != 48) {
            throw new Asn1Exception(906);
        }
        Vector<PAData> vector = new Vector();
        while (derValue.getData().available() > 0) {
            vector.addElement(new PAData(derValue.getData().getDerValue()));
        }
        if (vector.size() > 0) {
            PAData[] anArray = new PAData[vector.size()];
            vector.copyInto(anArray);
            return anArray;
        }
        return null;
    }
    
    public static int getPreferredEType(PAData[] array, int n) throws IOException, Asn1Exception {
        if (array == null) {
            return n;
        }
        Object o = null;
        Object o2 = null;
        for (PAData paData : array) {
            if (paData.getValue() != null) {
                switch (paData.getType()) {
                    case 11: {
                        o = new DerValue(paData.getValue());
                        break;
                    }
                    case 19: {
                        o2 = new DerValue(paData.getValue());
                        break;
                    }
                }
            }
        }
        if (o2 != null) {
            while (o2.data.available() > 0) {
                ETypeInfo2 o3 = new ETypeInfo2(o2.data.getDerValue());
                if (o3.getParams() == null) {
                    return o3.getEType();
                }
                continue;
            }
        }
        if (o != null && o.data.available() > 0) {
            return new ETypeInfo(o.data.getDerValue()).getEType();
        }
        return n;
    }
    
    public static PAData.SaltAndParams getSaltAndParams(int n, PAData[] array) throws Asn1Exception, IOException {
        if (array == null) {
            return null;
        }
        Object o = null;
        Object o2 = null;
        Object o3 = null;
        for (PAData paData : array) {
            if (paData.getValue() != null) {
                switch (paData.getType()) {
                    case 3: {
                        o3 = new String(paData.getValue(), KerberosString.MSNAME ? "UTF8" : "8859_1");
                        break;
                    }
                    case 11: {
                        o = new DerValue(paData.getValue());
                        break;
                    }
                    case 19: {
                        o2 = new DerValue(paData.getValue());
                        break;
                    }
                }
            }
        }
        if (o2 != null) {
            while (o2.data.available() > 0) {
                ETypeInfo2 o4 = new ETypeInfo2(o2.data.getDerValue());
                if (o4.getParams() == null && o4.getEType() == n) {
                    return new PAData.SaltAndParams(o4.getSalt(), o4.getParams());
                }
                continue;
            }
        }
        if (o != null) {
            while (o.data.available() > 0) {
                ETypeInfo o5 = new ETypeInfo(o.data.getDerValue());
                if (o5.getEType() == n) {
                    return new PAData.SaltAndParams(o5.getSalt(), null);
                }
                continue;
            }
        }
        if (o3 != null) {
            return new PAData.SaltAndParams(o3, null);
        }
        return null;
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(">>>Pre-Authentication Data:\n\t PA-DATA type = ").append(this.pADataType).append('\n');
        switch (this.pADataType) {
            case 2: {
                sb.append("\t PA-ENC-TIMESTAMP");
                break;
            }
            case 11: {
                if (this.pADataValue != null) {
                    try {
                        DerValue derValue = new DerValue(this.pADataValue);
                        while (derValue.data.available() > 0) {
                            ETypeInfo o = new ETypeInfo(derValue.data.getDerValue());
                            sb.append("\t PA-ETYPE-INFO etype = ").append(o.getEType()).append(", salt = ").append(o.getSalt()).append('\n');
                        }
                        break;
                    }
                    catch (IOException | Asn1Exception ex) {
                        sb.append("\t <Unparseable PA-ETYPE-INFO>\n");
                        break;
                    }
                    break;
                }
                break;
            }
            case 19: {
                if (this.pADataValue != null) {
                    try {
                        DerValue derValue2 = new DerValue(this.pADataValue);
                        while (derValue2.data.available() > 0) {
                            ETypeInfo2 o2 = new ETypeInfo2(derValue2.data.getDerValue());
                            sb.append("\t PA-ETYPE-INFO2 etype = ").append(o2.getEType()).append(", salt = ").append(o2.getSalt()).append(", s2kparams = ");
                            byte[] params = o2.getParams();
                            if (params == null) {
                                sb.append("null\n");
                            }
                            else if (params.length == 0) {
                                sb.append("empty\n");
                            }
                            else {
                                sb.append(new HexDumpEncoder().encodeBuffer(params));
                            }
                        }
                        break;
                    }
                    catch (IOException | Asn1Exception ex2) {
                        sb.append("\t <Unparseable PA-ETYPE-INFO>\n");
                        break;
                    }
                    break;
                }
                break;
            }
            case 129: {
                sb.append("\t PA-FOR-USER\n");
                break;
            }
        }
        return sb.toString();
    }
    
    public static class SaltAndParams
    {
        public final String salt;
        public final byte[] params;
        
        public SaltAndParams(String salt, byte[] params) {
            if (salt != null && salt.isEmpty()) {
                salt = null;
            }
            this.salt = salt;
            this.params = params;
        }
    }
}
